@@ -61,7 +61,7 @@ or the C<spamd>/C<spamc> tools provided.
 package Mail::SpamAssassin;
 use strict;
 use warnings;
-use bytes;
+# use bytes;
 use re 'taint';
 
 require 5.006_001;
@@ -87,37 +87,32 @@ use Time::HiRes qw(time);
 use Cwd;
 use Config;
 
-use vars qw{
-  @ISA $VERSION $SUB_VERSION @EXTRA_VERSION $IS_DEVEL_BUILD $HOME_URL
-  @default_rules_path @default_prefs_path
-  @default_userprefs_path @default_userstate_dir
-  @site_rules_path
-};
+our $VERSION = "3.004002";      # update after release (same format as perl $])
+#our $IS_DEVEL_BUILD = 1;        # 1 for devel build 
+our $IS_DEVEL_BUILD = 0;        # 0 for release versions including rc & pre releases
 
-$VERSION = "3.004001";      # update after release (same format as perl $])
-#$IS_DEVEL_BUILD = 1;        # change for release versions
 
 # Used during the prerelease/release-candidate part of the official release
 # process. If you hacked up your SA, you should add a version_tag to your .cf
 # files; this variable should not be modified.
-#@EXTRA_VERSION = qw();
+our @EXTRA_VERSION = qw();
 
-@ISA = qw();
+our @ISA = qw();
 
 # SUB_VERSION is now just <yyyy>-<mm>-<dd>
-$SUB_VERSION = 'svnunknown';
-if ('$LastChangedDate: 2015-04-28 03:54:45 -0400 (Tue, 28 Apr 2015) $' =~ ':') {
-  # Subversion keyword "$LastChangedDate: 2015-04-28 03:54:45 -0400 (Tue, 28 Apr 2015) $" has been successfully expanded.
+our $SUB_VERSION = 'svnunknown';
+if ('$LastChangedDate: 2018-09-13 21:25:10 -0400 (Thu, 13 Sep 2018) $' =~ ':') {
+  # Subversion keyword "$LastChangedDate: 2018-09-13 21:25:10 -0400 (Thu, 13 Sep 2018) $" has been successfully expanded.
   # Doesn't happen with automated launchpad builds:
   # https://bugs.launchpad.net/launchpad/+bug/780916
-  $SUB_VERSION = (split(/\s+/,'$LastChangedDate: 2015-04-28 03:54:45 -0400 (Tue, 28 Apr 2015) $ updated by SVN'))[1];
+  $SUB_VERSION = (split(/\s+/,'$LastChangedDate: 2018-09-13 21:25:10 -0400 (Thu, 13 Sep 2018) $ updated by SVN'))[1];
 }
 
 
 if (defined $IS_DEVEL_BUILD && $IS_DEVEL_BUILD) {
-  if ('$LastChangedRevision: 1676427 $' =~ ':') {
-    # Subversion keyword "$LastChangedRevision: 1676427 $" has been successfully expanded.
-    push(@EXTRA_VERSION, ('r' . qw{$LastChangedRevision: 1676427 $ updated by SVN}[1]));
+  if ('$LastChangedRevision: 1840870 $' =~ ':') {
+    # Subversion keyword "$LastChangedRevision: 1840870 $" has been successfully expanded.
+    push(@EXTRA_VERSION, ('r' . qw{$LastChangedRevision: 1840870 $ updated by SVN}[1]));
   } else {
     push(@EXTRA_VERSION, ('r' . 'svnunknown'));
   }
@@ -128,13 +123,14 @@ sub Version {
   return join('-', sprintf("%d.%d.%d", $1, $2, $3), @EXTRA_VERSION);
 }
 
-$HOME_URL = "http://spamassassin.apache.org/";
+our $HOME_URL = "http://spamassassin.apache.org/";
 
 # note that the CWD takes priority.  This is required in case a user
 # is testing a new version of SpamAssassin on a machine with an older
 # version installed.  Unless you can come up with a fix for this that
 # allows "make test" to work, don't change this.
-@default_rules_path = (
+
+our @default_rules_path = (
   '__local_state_dir__/__version__',
   '__def_rules_dir__',
   '__prefix__/share/spamassassin',
@@ -143,7 +139,7 @@ $HOME_URL = "http://spamassassin.apache.org/";
 );
 
 # first 3 are BSDish, latter 2 Linuxish
-@site_rules_path = (
+our @site_rules_path = (
   '__local_rules_dir__',
   '__prefix__/etc/mail/spamassassin',
   '__prefix__/etc/spamassassin',
@@ -154,21 +150,23 @@ $HOME_URL = "http://spamassassin.apache.org/";
   '/etc/spamassassin',
 );
 
-@default_prefs_path = (
+our @default_prefs_path = (
   '__local_rules_dir__/user_prefs.template',
   '__prefix__/etc/mail/spamassassin/user_prefs.template',
   '__prefix__/share/spamassassin/user_prefs.template',
+  '__local_state_dir__/__version__/updates_spamassassin_org/user_prefs.template',
+  '__def_rules_dir__/user_prefs.template',
   '/etc/spamassassin/user_prefs.template',
   '/etc/mail/spamassassin/user_prefs.template',
   '/usr/local/share/spamassassin/user_prefs.template',
   '/usr/share/spamassassin/user_prefs.template',
 );
 
-@default_userprefs_path = (
+our @default_userprefs_path = (
   '~/.spamassassin/user_prefs',
 );
 
-@default_userstate_dir = (
+our @default_userstate_dir = (
   '~/.spamassassin',
 );
 
@@ -840,7 +838,7 @@ sub signal_user_changed {
   $self->{'learn_to_journal'} = $self->{conf}->{bayes_learn_to_journal};
 
   $set |= 1 unless $self->{local_tests_only};
-  $set |= 2 if $self->{bayes_scanner} && $self->{bayes_scanner}->is_scan_available();
+  $set |= 2 if $self->{bayes_scanner} && $self->{bayes_scanner}->is_scan_available() && $self->{conf}->{use_bayes_rules};
 
   $self->{conf}->set_score_set ($set);
 
@@ -1777,7 +1775,7 @@ sub init {
   # Figure out/set our initial scoreset
   my $set = 0;
   $set |= 1 unless $self->{local_tests_only};
-  $set |= 2 if $self->{bayes_scanner} && $self->{bayes_scanner}->is_scan_available();
+  $set |= 2 if $self->{bayes_scanner} && $self->{bayes_scanner}->is_scan_available() && $self->{conf}->{use_bayes_rules};
   $self->{conf}->set_score_set ($set);
 
   if ($self->{only_these_rules}) {
