@@ -30,7 +30,7 @@ package Mail::SpamAssassin::BayesStore;
 
 use strict;
 use warnings;
-use bytes;
+# use bytes;
 use re 'taint';
 use Mail::SpamAssassin::Logger;
 
